import { Plugin, MountData, Next, InvokeData, UseifyPlugin } from '@faasjs/func';

type ValidatorConfig = {
    whitelist?: 'error' | 'ignore';
    rules: {
        [key: string]: {
            type?: 'string' | 'number' | 'boolean' | 'object' | 'array';
            required?: boolean;
            in?: any[];
            default?: any;
            config?: Partial<ValidatorConfig>;
        };
    };
    onError?: (type: string, key: string | string[], value?: any) => {
        statusCode?: number;
        headers?: {
            [name: string]: any;
        };
        message: any;
    } | void;
};

/**
 * A FaasJS plugin, let function could create, config and invoke CloudFunction.
 *
 * [![License: MIT](https://img.shields.io/npm/l/@faasjs/cloud_function.svg)](https://github.com/faasjs/faasjs/blob/main/packages/cloud_function/LICENSE)
 * [![NPM Version](https://img.shields.io/npm/v/@faasjs/cloud_function.svg)](https://www.npmjs.com/package/@faasjs/cloud_function)
 *
 * ## Install
 *
 * ```sh
 * npm install @faasjs/cloud_function
 * ```
 *
 * @packageDocumentation
 */

/** 云函数配置项 */
type CloudFunctionConfig = {
    /** 插件名称 */
    name?: string;
    /** 配置项 */
    config?: {
        /** 配置名称 */
        name?: string;
        /** 内存大小，单位为MB，默认 64 */
        memorySize?: 64 | 128 | 256 | 384 | 512 | 640 | 768 | 896 | 1024 | number;
        /** 执行超时时间，单位为秒，默认 30 */
        timeout?: number;
        /** 触发器配置 */
        triggers?: {
            type: 'timer' | string;
            name?: string;
            value: string;
        }[];
        /** 预制并发配置 */
        provisionedConcurrent?: {
            /** 预制并发数量 */
            executions: number;
        };
        [key: string]: any;
    };
    validator?: {
        event?: ValidatorConfig;
    };
    [key: string]: any;
};
type CloudFunctionAdapter = {
    invokeCloudFunction: (name: string, data: any, options?: any) => Promise<void>;
    invokeSyncCloudFunction: <TResult>(name: string, data: any, options?: any) => Promise<TResult>;
};
declare class CloudFunction implements Plugin {
    readonly type: string;
    readonly name: string;
    event: any;
    context: any;
    config: {
        name?: string;
        memorySize?: number;
        timeout?: number;
        triggers?: {
            type: string;
            name: string;
            value: string;
        }[];
        [key: string]: any;
    };
    private adapter;
    private readonly validatorConfig?;
    private validator?;
    private readonly logger;
    /**
     * 创建云函数配置
     * @param config {object} 配置项，这些配置将强制覆盖默认配置
     * @param config.name {string} 云资源名
     * @param config.config {object} 云资源配置
     * @param config.config.name {string} 云函数名
     * @param config.config.memorySize {number} 内存大小，单位为 MB
     * @param config.config.timeout {number} 最长执行时间，单位为 秒
     * @param config.config.triggers {object[]} 触发器配置
     * @param config.config.provisionedConcurrent {object} 预制并发配置
     * @param config.config.provisionedConcurrent.executions {number} 并发数
     * @param config.validator {object} 事件校验配置
     * @param config.validator.event {object} event 校验配置
     * @param config.validator.event.whitelist {string} 白名单配置
     * @param config.validator.event.onError {function} 自定义报错
     * @param config.validator.event.rules {object} 参数校验规则
     */
    constructor(config?: CloudFunctionConfig);
    onMount(data: MountData, next: Next): Promise<void>;
    onInvoke(data: InvokeData, next: Next): Promise<void>;
    /**
     * 异步触发云函数
     * @param name {string} 云函数文件名或云函数名
     * @param data {any} 参数
     * @param options {object} 额外配置项
     */
    invoke<TData = any>(name: string, data?: TData, options?: Record<string, any>): Promise<void>;
    /**
     * 同步调用云函数
     * @param name {string} 云函数文件名或云函数名
     * @param data {any} 参数
     * @param options {object} 额外配置项
     */
    invokeSync<TResult = any, TData = any>(name: string, data?: TData, options?: Record<string, any>): Promise<TResult>;
}
declare function useCloudFunction(config?: CloudFunctionConfig | (() => CloudFunctionConfig)): UseifyPlugin<CloudFunction>;
/**
 * 异步触发云函数
 * @param name {string} 云函数文件名或云函数名
 * @param data {any} 参数
 * @param options {object} 额外配置项
 */
declare function invoke<TData = any>(name: string, data?: TData, options?: {
    [key: string]: any;
}): Promise<void>;
/**
 * 同步触发云函数
 * @param name {string} 云函数文件名或云函数名
 * @param data {any} 参数
 * @param options {object} 额外配置项
 */
declare function invokeSync<TResult = any, TData = any>(name: string, data?: TData, options?: {
    [key: string]: any;
}): Promise<TResult>;

export { CloudFunction, type CloudFunctionAdapter, type CloudFunctionConfig, invoke, invokeSync, useCloudFunction };
