import { deepMerge } from '@faasjs/deep_merge';
import { usePlugin } from '@faasjs/func';
import { loadPackage } from '@faasjs/load';
import { Logger } from '@faasjs/logger';

// src/index.ts
var Validator = class {
  eventConfig;
  request;
  logger;
  constructor(config) {
    this.eventConfig = config.event;
    this.logger = new Logger("CloudFunction.Validator");
  }
  valid({ event }) {
    this.request = { event };
    this.logger.debug("Begin");
    if (this.eventConfig) {
      this.logger.debug("Valid event");
      this.validContent("event", event, "", this.eventConfig);
    }
  }
  validContent(type, event, baseKey, config) {
    if (config.whitelist) {
      const eventKeys = Object.keys(event);
      const rulesKeys = Object.keys(config.rules).concat(["context"]);
      const diff = eventKeys.filter((k) => !rulesKeys.includes(k));
      if (diff.length > 0) {
        if (config.whitelist === "error") {
          const diffKeys = diff.map((k) => `${baseKey}${k}`);
          const error = Error(
            `[${type}] Not permitted keys: ${diffKeys.join(", ")}`
          );
          if (config.onError)
            config.onError(`${type}.whitelist`, baseKey, diffKeys);
          throw error;
        }
        if (config.whitelist === "ignore")
          for (const key of diff) delete event[key];
      }
    }
    for (const key in config.rules) {
      const rule = config.rules[key];
      let value = event[key];
      if (rule.default) {
        if (type === "cookie" || type === "session")
          this.logger.warn("Cookie and Session not support default rule.");
        else if (typeof value === "undefined" && rule.default) {
          value = typeof rule.default === "function" ? rule.default(this.request) : rule.default;
          event[key] = value;
        }
      }
      if (rule.required) {
        if (typeof value === "undefined" || value === null) {
          const error = Error(`[${type}] ${baseKey}${key} is required.`);
          if (config.onError)
            config.onError(`${type}.rule.required`, `${baseKey}${key}`, value);
          throw error;
        }
      }
      if (typeof value !== "undefined" && value !== null) {
        if (rule.type)
          if (type === "cookie")
            this.logger.warn("Cookie not support type rule");
          else {
            let typed = true;
            switch (rule.type) {
              case "array":
                typed = Array.isArray(value);
                break;
              case "object":
                typed = Object.prototype.toString.call(value) === "[object Object]";
                break;
              default:
                typed = typeof value === rule.type;
                break;
            }
            if (!typed) {
              const error = Error(
                `[${type}] ${baseKey}${key} must be a ${rule.type}.`
              );
              if (config.onError)
                config.onError(`${type}.rule.type`, `${baseKey}${key}`, value);
              throw error;
            }
          }
        if (rule.in && !rule.in.includes(value)) {
          const error = Error(
            `[${type}] ${baseKey}${key} must be in ${rule.in.join(", ")}.`
          );
          if (config.onError)
            config.onError(`${type}.rule.in`, `${baseKey}${key}`, value);
          throw error;
        }
        if (rule.config) {
          if (type === "cookie")
            this.logger.warn("Cookie not support nest rule.");
          else if (Array.isArray(value))
            for (const val of value)
              this.validContent(
                type,
                val,
                baseKey ? `${baseKey}.${key}.` : `${key}.`,
                rule.config
              );
          else if (typeof value === "object")
            this.validContent(
              type,
              value,
              baseKey ? `${baseKey}.${key}.` : `${key}.`,
              rule.config
            );
        }
      }
    }
  }
};

// src/index.ts
var Name = "cloud_function";
var globals = {};
var CloudFunction = class {
  type = Name;
  name = Name;
  event;
  context;
  config;
  adapter;
  validatorConfig;
  validator;
  logger;
  /**
   * 创建云函数配置
   * @param config {object} 配置项，这些配置将强制覆盖默认配置
   * @param config.name {string} 云资源名
   * @param config.config {object} 云资源配置
   * @param config.config.name {string} 云函数名
   * @param config.config.memorySize {number} 内存大小，单位为 MB
   * @param config.config.timeout {number} 最长执行时间，单位为 秒
   * @param config.config.triggers {object[]} 触发器配置
   * @param config.config.provisionedConcurrent {object} 预制并发配置
   * @param config.config.provisionedConcurrent.executions {number} 并发数
   * @param config.validator {object} 事件校验配置
   * @param config.validator.event {object} event 校验配置
   * @param config.validator.event.whitelist {string} 白名单配置
   * @param config.validator.event.onError {function} 自定义报错
   * @param config.validator.event.rules {object} 参数校验规则
   */
  constructor(config) {
    if (config) {
      this.name = config.name || Name;
      this.config = config.config || /* @__PURE__ */ Object.create(null);
      if (config.validator) this.validatorConfig = config.validator;
    } else {
      this.name = this.type;
      this.config = /* @__PURE__ */ Object.create(null);
    }
    this.logger = new Logger(this.name);
  }
  async onMount(data, next) {
    if (data.config.plugins?.[this.name || this.type])
      this.config = deepMerge(
        { config: this.config },
        data.config.plugins[this.name || this.type],
        {}
      );
    if (this.config.provider) {
      const Provider = (await import(this.config.provider.type)).Provider;
      this.adapter = new Provider(this.config.provider.config);
    } else
      this.logger.warn(
        "[onMount] Unknown provider, will use invoke and invokeSync with local mode."
      );
    if (this.validatorConfig) {
      this.logger.debug("[onMount] prepare validator");
      this.validator = new Validator(this.validatorConfig);
    }
    globals[this.name] = this;
    await next();
  }
  async onInvoke(data, next) {
    this.event = data.event;
    this.context = data.context;
    if (this.validator) {
      this.logger.debug("[onInvoke] Valid");
      this.validator.valid({ event: this.event });
    }
    await next();
  }
  /**
   * 异步触发云函数
   * @param name {string} 云函数文件名或云函数名
   * @param data {any} 参数
   * @param options {object} 额外配置项
   */
  async invoke(name, data, options) {
    if (data == null) data = /* @__PURE__ */ Object.create(null);
    if (process.env.FaasMode !== "remote") {
      const test = await loadPackage("@faasjs/test");
      const originFunc = await loadPackage(
        `${process.env.FaasRoot || ""}${name.toLowerCase()}.func`,
        ["func", "default"]
      );
      const func = new test.FuncWarper(originFunc);
      return func.handler(data, { request_id: this.logger.label });
    }
    return this.adapter.invokeCloudFunction(name.toLowerCase(), data, options);
  }
  /**
   * 同步调用云函数
   * @param name {string} 云函数文件名或云函数名
   * @param data {any} 参数
   * @param options {object} 额外配置项
   */
  async invokeSync(name, data, options) {
    if (data == null) data = /* @__PURE__ */ Object.create(null);
    if (process.env.FaasMode !== "remote") {
      const test = await loadPackage("@faasjs/test");
      const originFunc = await loadPackage(
        `${process.env.FaasRoot || ""}${name.toLowerCase()}.func`,
        ["func", "default"]
      );
      const func = new test.FuncWarper(originFunc);
      return func.handler(data, { request_id: this.logger.label });
    }
    return this.adapter.invokeSyncCloudFunction(
      name.toLowerCase(),
      data,
      options
    );
  }
};
function useCloudFunction(config) {
  let configs;
  if (config)
    if (typeof config === "function") configs = config();
    else configs = config;
  const name = configs?.name || Name;
  if (globals[name]) return usePlugin(globals[name]);
  return usePlugin(new CloudFunction(configs));
}
async function invoke(name, data, options) {
  return await useCloudFunction().invoke(name, data, options);
}
async function invokeSync(name, data, options) {
  return await useCloudFunction().invokeSync(
    name,
    data,
    options
  );
}

export { CloudFunction, invoke, invokeSync, useCloudFunction };
